package com.fs.statis.service.impl;

import com.fs.company.domain.CompanyUser;
import com.fs.company.mapper.CompanyUserMapper;
import com.fs.qw.mapper.QwUserMapper;
import com.fs.sop.mapper.QwSopLogsMapper;
import com.fs.statis.domain.FsStatisSalerWatch;
import com.fs.statis.dto.StatsWatchLogPageListDTO;
import com.fs.statis.mapper.FsStatisSalerWatchMapper;
import com.fs.statis.service.FsStatisSalerWatchService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 销售观看统计服务实现类
 */
@Service
public class FsStatisSalerWatchServiceImpl implements FsStatisSalerWatchService {

    private final FsStatisSalerWatchMapper fsStatisSalerWatchMapper;

    private final CompanyUserMapper companyUserMapper;

    private final QwUserMapper qwUserMapper;

    private final QwSopLogsMapper qwSopLogsMapper;

    @Autowired
    public FsStatisSalerWatchServiceImpl(FsStatisSalerWatchMapper fsStatisSalerWatchMapper, CompanyUserMapper companyUserMapper, QwUserMapper qwUserMapper, QwSopLogsMapper qwSopLogsMapper) {
        this.fsStatisSalerWatchMapper = fsStatisSalerWatchMapper;
        this.companyUserMapper = companyUserMapper;
        this.qwUserMapper = qwUserMapper;
        this.qwSopLogsMapper = qwSopLogsMapper;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public FsStatisSalerWatch getById(Integer id) {
        if (id == null) {
            return null;
        }
        return fsStatisSalerWatchMapper.findById(id);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<FsStatisSalerWatch> getAll() {
        return fsStatisSalerWatchMapper.findAll();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public FsStatisSalerWatch create(FsStatisSalerWatch fsStatisSalerWatch) {
        if (fsStatisSalerWatch == null) {
            return null;
        }
        fsStatisSalerWatch.setId(null);
        fsStatisSalerWatchMapper.insert(fsStatisSalerWatch);
        return fsStatisSalerWatch;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public FsStatisSalerWatch update(FsStatisSalerWatch fsStatisSalerWatch) {
        if (fsStatisSalerWatch == null || fsStatisSalerWatch.getId() == null) {
            return null;
        }
        int affectedRows = fsStatisSalerWatchMapper.update(fsStatisSalerWatch);
        if (affectedRows > 0) {
            return fsStatisSalerWatch;
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean deleteById(Integer id) {
        if (id == null) {
            return false;
        }
        int affectedRows = fsStatisSalerWatchMapper.deleteById(id);
        return affectedRows > 0;
    }

    @Override
    public List<FsStatisSalerWatch> queryList(StatsWatchLogPageListDTO param) {
        return fsStatisSalerWatchMapper.queryList(param);
    }

    @Override
    public void writeData() {
        // 统计销售看课情况
        // 获取前一天的时间
        List<CompanyUser> companyUserList = this.companyUserMapper.selectAllCompanyUserList();

        for (CompanyUser companyUser : companyUserList) {
            // 找到销售关联的企微账号
            List<String> qwUserIdList = qwUserMapper.findQwUserIdListByCompanyUserId(companyUser.getUserId());
            // 去sop记录表找对应的SOP发送记录，记录数作为营期人数
            Long periodCount = qwSopLogsMapper.selectQwSopLogsCountByQwUserId(qwUserIdList);
            // 再去course_watch_log找对应的销售观看记录作为已报名数
            Long registerCount = companyUserMapper.queryCompanyUserWatchCount(companyUser.getUserId());
            Long completedCount = companyUserMapper.queryCompanyUserWatchCountCompleted(companyUser.getUserId());


            FsStatisSalerWatch fsStatisSalerWatch = new FsStatisSalerWatch();
            fsStatisSalerWatch.setDeptId(companyUser.getDeptId());
            fsStatisSalerWatch.setCompanyUserId(companyUser.getUserId());
            fsStatisSalerWatch.setTrainCampNum(periodCount);
            fsStatisSalerWatch.setNotRegisteredNum(periodCount - registerCount);
            fsStatisSalerWatch.setRegisteredNum(registerCount);
            fsStatisSalerWatch.setCompletedNum(completedCount);
            fsStatisSalerWatch.setFinishedRate((float) (completedCount / registerCount));
            fsStatisSalerWatch.setRegRate((float) (registerCount / periodCount));

        }
    }
}
